module PlainHaskellExample where

data Database = Database

type Person = String

type Result = Either String (Maybe (Int, Maybe (Either Int Int)))

salaryOf :: Person -> Database -> Result
salaryOf _ _ = undefined

-- some example results
result1 :: Result
result1 = Left "Person not found"

result2 :: Result
result2 = Left "Person not unique"

result3 :: Result
result3 = Right (Just (1000, Just (Left 50)))

result4 :: Result
result4 = Right Nothing

result5 :: Result
result5 = Right (Just (1000, Nothing))

result6 :: Result
result6 = Right (Just (2000, Just (Right 500)))


-- re-modelled:

data Fallible a = Failed String | Okay a

data Salary = NoSalary | Salary Int Bonus

data Bonus = NoBonus | MonthlyBonus Int | YearlyBonus Int

type Result' = Fallible Salary

result1' :: Result'
result1' = Failed "Person not found"

result2' :: Result'
result2' = Failed "Person not unique"

result3' :: Result'
result3' = Okay (Salary 1000 (MonthlyBonus 50))

result4' :: Result'
result4' = Okay NoSalary

result5' :: Result'
result5' = Okay (Salary 1000 NoBonus)

result6' :: Result'
result6' = Okay (Salary 2000 (YearlyBonus 500))
